{% extends 'base.html' %}

{% block styling %}
<style>
    form {
        width: 100vw;
        display: flex;
        align-items: center;
        flex-direction: column;
    }

    input[type=text] {
        width: calc(15rem + 20vw);
    }
</style>
{% endblock %}

{% block content %}
<h1 class="hero">yark</h1>
<form action="/" method="post">
    <input type="text" name="channel" placeholder="e.g. pewdiepie">
</form>
{% endblock %}